#ifndef __NOWGG_SDK_H__
#define __NOWGG_SDK_H__

#include <string>
#include <vector>

using namespace std;

namespace nowgg
{
    /*
    * model for getting purchasable item details
    */
    struct SkuDetails
    {
        SkuDetails(): productId(""), type("inapp"), description(""),
        introductoryPrice(""), introductoryPriceAmountMicros(""), introductoryPriceCycles(""),
        introductoryPricePeriod(""), price(""), priceAmountMicros(0), priceCurrencyCode(""),
        rewarded(false), subscriptionPeriod(""), title("") {}
        
        string productId;
        string type;
        string description;
        string introductoryPrice;
        string introductoryPriceAmountMicros;
        string introductoryPriceCycles;
        string introductoryPricePeriod;
        string price;
        double priceAmountMicros;
        string priceCurrencyCode;
        bool rewarded;
        string subscriptionPeriod;
        string title;
    };

    /*
    * model for purchased item
    */
    struct Product
    {
        Product() : autoRenewing(false), developerPayload(""), orderId(""), packageName(""),
        productId(""), purchaseTime(0), purchaseToken(""),
        signature(""), originalJson("") {}

        bool autoRenewing;
        string developerPayload;
        string orderId;
        string packageName;
        string productId;
        double purchaseTime;
        string purchaseToken;
        string signature;
        string originalJson;
    };

    /**
    * The listener for NowGG IAP
    */
    class NowGGSDKListener
    {
    public:
        /**
        * Called when IAP service is initialized with success or failure
        */
        virtual void onInitialized(bool success) = 0;

        /**
        * Called when an Item is purchased successfully, this contains list of all the items thats are in purchased state
        */
        virtual void onPurchaseUpdated(const vector<Product*> p) = 0;

        /**
        * Called when purchase of item is cancelled
        */
        virtual void onPurchaseCancelled(int code, string msg) = 0;

        /**
        * Called when an item is consumed successfully
        */
        virtual void onPurchaseConsumed(string token) = 0;

        /**
        * Called when login is successful
        */
        virtual void onLoginSuccess(string email, string token, string hostUrl) = 0;

        /**
        * Called when login is failed
        */
        virtual void onLoginFailed(int code, string errorMsg) = 0;

    };

    class NowGGSDK
    {
    public:
        /**
        *returns true when nowgg IAP is available else returns false
        */
        static bool isNowGGIapAvailable();

        /**
        *enable logcat for NowGG sdk for debuggin purpose
        */
        static void setDebug(bool debug);
        
        /**
        *initialize nowgg sdk with app id and vector of items
        */
        static void init(string appId, vector<string> productId);

        /**
        * @brief Get the Product Details object
        * 
        * @return * Get 
        */
        static vector<SkuDetails*> getProductDetails();

        /**
        * Make a purchase request, type needs to be inapp for IAP and subs for subscription
        */
        static void purchase(string name, string type);

        /**
        * Make a consume purchase request
        */
        static void consume(string token);
        
        /**
        * Refresh purchases
        */
        static void refreshPurchases();

        /**
        * Set listener for IAP
        */
        static void setListener(NowGGSDKListener* listener);

        /**
        * Remove listener for IAP
        */
        static void removeListener();

        /**
        * login
        */
		static void login(string clientId);
        static void login(string clientId, string idToken);

        /**
        * add NowGG account if login is not present
        */
        static void addNowGGAccount();
    };
}
#endif // __NOWGG_SDK_H__
